@@ -32,15 +32,18 @@ class WN(torch.nn.Module):
         dropout_p (float): dropout rate.
         weight_norm (bool): enable/disable weight norm for convolution layers.
     """
-    def __init__(self,
-                 in_channels,
-                 hidden_channels,
-                 kernel_size,
-                 dilation_rate,
-                 num_layers,
-                 c_in_channels=0,
-                 dropout_p=0,
-                 weight_norm=True):
+
+    def __init__(
+        self,
+        in_channels,
+        hidden_channels,
+        kernel_size,
+        dilation_rate,
+        num_layers,
+        c_in_channels=0,
+        dropout_p=0,
+        weight_norm=True,
+    ):
         super().__init__()
         assert kernel_size % 2 == 1
         assert hidden_channels % 2 == 0
@@ -58,20 +61,16 @@ class WN(torch.nn.Module):
 
         # init conditioning layer
         if c_in_channels > 0:
-            cond_layer = torch.nn.Conv1d(c_in_channels,
-                                         2 * hidden_channels * num_layers, 1)
-            self.cond_layer = torch.nn.utils.weight_norm(cond_layer,
-                                                         name='weight')
+            cond_layer = torch.nn.Conv1d(c_in_channels, 2 * hidden_channels * num_layers, 1)
+            self.cond_layer = torch.nn.utils.weight_norm(cond_layer, name="weight")
         # intermediate layers
         for i in range(num_layers):
-            dilation = dilation_rate**i
+            dilation = dilation_rate ** i
             padding = int((kernel_size * dilation - dilation) / 2)
-            in_layer = torch.nn.Conv1d(hidden_channels,
-                                       2 * hidden_channels,
-                                       kernel_size,
-                                       dilation=dilation,
-                                       padding=padding)
-            in_layer = torch.nn.utils.weight_norm(in_layer, name='weight')
+            in_layer = torch.nn.Conv1d(
+                hidden_channels, 2 * hidden_channels, kernel_size, dilation=dilation, padding=padding
+            )
+            in_layer = torch.nn.utils.weight_norm(in_layer, name="weight")
             self.in_layers.append(in_layer)
 
             if i < num_layers - 1:
@@ -79,10 +78,8 @@ class WN(torch.nn.Module):
             else:
                 res_skip_channels = hidden_channels
 
-            res_skip_layer = torch.nn.Conv1d(hidden_channels,
-                                             res_skip_channels, 1)
-            res_skip_layer = torch.nn.utils.weight_norm(res_skip_layer,
-                                                        name='weight')
+            res_skip_layer = torch.nn.Conv1d(hidden_channels, res_skip_channels, 1)
+            res_skip_layer = torch.nn.utils.weight_norm(res_skip_layer, name="weight")
             self.res_skip_layers.append(res_skip_layer)
         # setup weight norm
         if not weight_norm:
@@ -91,6 +88,7 @@ class WN(torch.nn.Module):
     def forward(self, x, x_mask=None, g=None, **kwargs):  # pylint: disable=unused-argument
         output = torch.zeros_like(x)
         n_channels_tensor = torch.IntTensor([self.hidden_channels])
+        x_mask = 1.0 if x_mask is None else x_mask
         if g is not None:
             g = self.cond_layer(g)
         for i in range(self.num_layers):
@@ -98,15 +96,14 @@ class WN(torch.nn.Module):
             x_in = self.dropout(x_in)
             if g is not None:
                 cond_offset = i * 2 * self.hidden_channels
-                g_l = g[:, cond_offset:cond_offset + 2 * self.hidden_channels, :]
+                g_l = g[:, cond_offset : cond_offset + 2 * self.hidden_channels, :]
             else:
                 g_l = torch.zeros_like(x_in)
-            acts = fused_add_tanh_sigmoid_multiply(x_in, g_l,
-                                                   n_channels_tensor)
+            acts = fused_add_tanh_sigmoid_multiply(x_in, g_l, n_channels_tensor)
             res_skip_acts = self.res_skip_layers[i](acts)
             if i < self.num_layers - 1:
-                x = (x + res_skip_acts[:, :self.hidden_channels, :]) * x_mask
-                output = output + res_skip_acts[:, self.hidden_channels:, :]
+                x = (x + res_skip_acts[:, : self.hidden_channels, :]) * x_mask
+                output = output + res_skip_acts[:, self.hidden_channels :, :]
             else:
                 output = output + res_skip_acts
         return output * x_mask
@@ -139,31 +136,35 @@ class WNBlocks(nn.Module):
         weight_norm (bool): enable/disable weight norm for convolution layers.
     """
 
-    def __init__(self,
-                 in_channels,
-                 hidden_channels,
-                 kernel_size,
-                 dilation_rate,
-                 num_blocks,
-                 num_layers,
-                 c_in_channels=0,
-                 dropout_p=0,
-                 weight_norm=True):
+    def __init__(
+        self,
+        in_channels,
+        hidden_channels,
+        kernel_size,
+        dilation_rate,
+        num_blocks,
+        num_layers,
+        c_in_channels=0,
+        dropout_p=0,
+        weight_norm=True,
+    ):
 
         super().__init__()
         self.wn_blocks = nn.ModuleList()
         for idx in range(num_blocks):
-            layer = WN(in_channels=in_channels if idx == 0 else hidden_channels,
-                       hidden_channels=hidden_channels,
-                       kernel_size=kernel_size,
-                       dilation_rate=dilation_rate,
-                       num_layers=num_layers,
-                       c_in_channels=c_in_channels,
-                       dropout_p=dropout_p,
-                       weight_norm=weight_norm)
+            layer = WN(
+                in_channels=in_channels if idx == 0 else hidden_channels,
+                hidden_channels=hidden_channels,
+                kernel_size=kernel_size,
+                dilation_rate=dilation_rate,
+                num_layers=num_layers,
+                c_in_channels=c_in_channels,
+                dropout_p=dropout_p,
+                weight_norm=weight_norm,
+            )
             self.wn_blocks.append(layer)
 
-    def forward(self, x, x_mask, g=None):
+    def forward(self, x, x_mask=None, g=None):
         o = x
         for layer in self.wn_blocks:
             o = layer(o, x_mask, g)
